package Bluestem::CGIClient;
use strict;
use warnings;
use FileHandle ();
use CGI ();
use URI::Escape qw(uri_escape uri_unescape);
use HTML::Template ();

use Bluestem::Client ();
our @ISA = qw(Bluestem::Client);

sub __remote_addr : method
{
	my $self = shift;
	my $addr = $ENV{REMOTE_ADDR};
	if( $ENV{HTTP_X_FORWARDED_FOR}
	 and $self->{conf}{proxy_forwarders}
	 and $self->{conf}{proxy_forwarders}{$addr}
	) {
		$addr = $ENV{HTTP_X_FORWARDED_FOR};
	}
	return $addr;
}

sub new : method
{
	my($class,%p) = @_;
	my $self = Bluestem::Client::new($class,%p);
	$self->{cgi} = $p{cgi} || CGI->new();
	return $self;
}

sub get_session : method
{
	my $self = shift;
	$self->{session} = undef;
	$self->get_session_noaccessupdate()
		or return;
	$self->update_cacheaccess( $self->{session}{cacheid} );
	return $self->{session};
}

sub get_session_noaccessupdate : method
{
	my $self = shift;
	my($cacheid,$clientkey,$paservers) = $self->read_sessioncookies();
	$self->{paservers} = $paservers;
	$self->{session} = $self->__get_session($cacheid,$clientkey,$self->__remote_addr)
		if $cacheid;
	return $self->{session};
}

sub read_sessioncookies : method
{
	my $self = shift;
	my $cookie = $self->{cgi}->cookie( $self->{conf}{client_cookiename} );
        $cookie =~ s/^"|"$//g if $cookie;
	my($cacheid,$clientkey) = split( /~/, $cookie )
		if $cookie;
	my %paservers = ();
	for my $idserver ( keys %{$self->{conf}{id_servers}} ) {
		my $cookiename = "Bluestem_$idserver";
		$cookiename =~ tr/:/_/;
		my $cookie = $self->{cgi}->cookie($cookiename)
			or next;
		$paservers{$idserver} = $cookie;
	}
	return($cacheid,$clientkey,\%paservers);
}

sub handle_login : method
{
	my $self = shift;
	my %p = @_;

	my $session = $p{session} || $self->get_session();
	my $paservers = $p{paservers} || $self->{paservers};
	my $iserver = $p{iserver} || $self->choose_id_server( $paservers );
	my $return_url = $p{return_url} || $self->{cgi}->url(-path_info=>1,-query=>1);
	my $message = $p{message};
	my $prior = $p{prior};

	# Remove old cache entry before redirect
	$self->remove_cacheentry( $session->{cacheid} )
		if $session;

	# Create new cache entry
	my($cacheid,$clientkey) = $self->create_cacheentry($iserver,$self->__remote_addr);

	my $login_url = $self->login_url(
		cacheid => $cacheid,
		iserver => $iserver,
		message => $message,
		return_url => $return_url,
		prior => $prior,
	);
	my $cookiestr = $self->client_cookie($cacheid,$clientkey);
	my $redirect = $self->{cgi}->redirect(
		-uri=>$login_url,
		-cookie=>[$cookiestr],
		-expires => "-1d",
	);
	print $redirect;
}

sub handle_retrieve_auth : method
{
	my $self = shift;
	my %p = @_;

	eval {
		my($cacheid,$clientkey,$paservers) = $self->read_sessioncookies();
		$cacheid or die "Error: No client cookie found!\n";
		my $auth = $self->retrieve_auth(cacheid=>$cacheid,clientip=>$self->__remote_addr,%p);
		my $return = $auth->{returnURL};
		$return =~ s{^https?://}{};
		my $cookiestr = $self->client_cookie($cacheid,$auth->{loginToken});
		print $self->{cgi}->redirect(
			-uri=> "https://$return",
			-cookie=>[$cookiestr],
			-expires => "-1d",
		);
	};
	if( $@ ) {
		$self->print_template("error.html",{ error => $@ });
	}
}

sub end_session : method
{
	my($self,$session) = @_;
	my %cookie_param = (
		-name    => "$self->{conf}{client_cookiename}",
		-value   => "",
		-secure  => 1,
	);
	$cookie_param{'-domain'} = $self->{conf}{client_cookiedomain}
		if $self->{conf}{client_cookiedomain} =~ m/^\./;
	my $cookie = $self->{cgi}->cookie(%cookie_param);
	$session ||= $self->get_session()
		or return;
	$self->remove_cacheentry( $session->{cacheid} );
	return $cookie;
}

sub handle_logout : method
{
	my($self,%p) = @_;
	
	my $session = $self->get_session()
		or return;
	my $paservers = $self->{paservers};
	if( %$paservers ) {
		my($iserver) = keys %$paservers;
		my $idlogout_url = $self->idlogout_url(
			iserver => $iserver,
			return_url => $self->{cgi}->self_url(),
		);
		print $self->{cgi}->redirect(
			-uri=>$idlogout_url,
			-expires => "-1d",
		);
	} else {
		my $cookie = $self->end_session( $session );
		if( $p{redirect} ) {
			print $self->{cgi}->redirect(
				-uri=>$p{return},
				-expires => "-1d",
				-cookie => $cookie,
			);
		} elsif( $p{template} ) {
			$self->print_template($p{template},{%p});
		}
	}
	return 1;
}

sub print_template : method
{
	my($self,$template,$param) = @_;
	$param ||= {};
	$template =~ m{/} and die "Invalid template specified!";
	my $tmpl = HTML::Template->new(
		filename => "$self->{conf}{template_dir}/$template",
		die_on_bad_params => 0,
	);
	for my $k ( keys %{$self->{conf}} ) {
		next if $k eq "server_key";
		$param->{$k} = $self->{conf}{$k};
	}
	$param->{REMOTE_ADDR} = $self->__remote_addr;
	$param->{REMOTE_HOST} = $ENV{REMOTE_HOST};
	$tmpl->param($param);
	print $self->{cgi}->header(
		-type=>"text/html",
		-expires => "-1d",
	);
	print $tmpl->output();
}

1;
